// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifndef MathMLOperatorElement_hh
#define MathMLOperatorElement_hh

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CharMap.hh"
#include "OperatorDefaults.hh"
#include "MathMLTokenElement.hh"

class MathMLOperatorElement: public MathMLTokenElement {
public:
  MathMLOperatorElement(void);
  virtual const AttributeSignature* GetAttributeSignature(AttributeId) const;
  virtual void Setup(class RenderingEnvironment*);
  virtual void Normalize(void);
  virtual ~MathMLOperatorElement();

  virtual bool IsOperator(void) const;
  virtual bool IsBreakable(void) const;
  bool         IsStretchy(void) const { return stretchy; }
  StretchId    GetStretch(void) const;
  bool         IsAccent(void) const { return accent; }
  bool         IsSeparator(void) const { return separator; }
  bool         HasMovableLimits(void) const { return movableLimits; }
  scaled       GetLeftPadding(void) const { return lSpace; }
  scaled       GetRightPadding(void) const { return rSpace; }

  void         HorizontalStretchTo(scaled, bool = false);
  void         VerticalStretchTo(scaled, scaled, bool = false);

private:
  const String* FlattenContent(void) const;
  OperatorFormId InferOperatorForm(void) const;
  const Value* GetOperatorAttributeValue(AttributeId,
					 const class RenderingEnvironment*) const;
  void ParseLimitValue(const Value*, const class RenderingEnvironment*, float&, scaled&);

  MathMLElement*          eOp; // ptr to the root of the embellished operator
  const String*           operatorName;
  OperatorFormId          form;
  const OperatorDefaults* defaults;

  scaled axis;

  bool 	 fence;
  bool 	 separator;
  scaled lSpace;
  scaled rSpace;
  bool   stretchy;
  bool 	 symmetric;
  bool 	 infiniteMaxSize; // if true => no max size contraint
  float  maxMultiplier;   // if <0 => absolute max size contraint
  scaled maxSize;
  float  minMultiplier;   // if <0 => absolute min size constraint
  scaled minSize;
  // bool 	 largeOp;
  bool 	 movableLimits;
  bool 	 accent;
};

typedef MathMLOperatorElement* MathMLOperatorElementPtr;

#define TO_OPERATOR(node) (dynamic_cast<MathMLOperatorElement*>(node))

#endif
