// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stddef.h>
#include <assert.h>

#include "OperatorDefaults.hh"

OperatorDefaults::OperatorDefaults()
{
  next = NULL;
}

OperatorDefaults::~OperatorDefaults()
{
  delete form;
  delete fence;
  delete separator;
  delete lspace;
  delete rspace;
  delete stretchy;
  delete direction;
  delete symmetric;
  delete maxsize;
  delete minsize;
  delete largeop;
  delete movablelimits;
  delete accent;

  next = NULL;
}

bool OperatorDefaults::Equal(const OperatorDefaults* op) const
{
  assert(op != NULL);

  if (!EqualAttribute(form, op->form)) return false;
  if (!EqualAttribute(fence, op->fence)) return false;
  if (!EqualAttribute(separator, op->separator)) return false;
  if (!EqualAttribute(lspace, op->lspace)) return false;
  if (!EqualAttribute(rspace, op->rspace)) return false;
  if (!EqualAttribute(stretchy, op->stretchy)) return false;
  if (!EqualAttribute(direction, op->direction)) return false;
  if (!EqualAttribute(symmetric, op->symmetric)) return false;
  if (!EqualAttribute(maxsize, op->maxsize)) return false;
  if (!EqualAttribute(minsize, op->minsize)) return false;
  if (!EqualAttribute(largeop, op->largeop)) return false;
  if (!EqualAttribute(movablelimits, op->movablelimits)) return false;
  if (!EqualAttribute(accent, op->accent)) return false;

  return true;
}

bool OperatorDefaults::EqualAttribute(const String* a, const String* b)
{
  if (a == NULL && b == NULL) return true;
  if (a == NULL && b != NULL) return false;
  if (a != NULL && b == NULL) return false;

  return a->Equal(*b);
}

